﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Caching;

namespace System.Web.AutoUpdateCache
{
    public delegate object UpdateCacheStart(object obj);

    public class AutoUpdateCache
    {
        private string name;

        private UpdateCacheStart start;

        private int minutes;

        private object obj;

        public static void Create(string name, int minutes, UpdateCacheStart start, object obj)
        {
            if (IsExcits(name)) throw new Exception("当前名称已经存在");
            if (minutes == 0) throw new Exception("时间应大于0");
            var update = new AutoUpdateCache(name, minutes, start, obj);
            update.Insert();
        }

        public static void Dispose(string name)
        {
            HttpRuntime.Cache.Remove(name);
        }

        public static void Remove(string name)
        {
            HttpRuntime.Cache.Remove(name);
        }

        public static object GetObject(string name)
        {
            return HttpRuntime.Cache[name];
        }

        public static bool IsExcits(string name)
        {
            return HttpRuntime.Cache[name] != null;
        }

        private AutoUpdateCache(string name, int minutes, UpdateCacheStart start, object obj)
        {
            this.name = name;
            this.minutes = minutes;
            this.start = start;
            this.obj = obj;
        }

        private void Insert()
        {
            HttpRuntime.Cache.Insert(name, Invoke(), null, DateTime.Now.AddMinutes(minutes), Cache.NoSlidingExpiration, new CacheItemUpdateCallback(Update));
        }

        private void Update(string key, CacheItemUpdateReason reason, out object expensiveObject, out CacheDependency dependency, out DateTime absoluteExpiration, out TimeSpan slidingExpiration)
        {
            expensiveObject = Invoke();
            dependency = null;
            absoluteExpiration = DateTime.Now.AddMinutes(minutes);
            slidingExpiration = Cache.NoSlidingExpiration;
        }

        private object Invoke()
        {
            var data = start.Invoke(obj);
            return data == null ? "Null" : data;
        }
    }

}
