﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

namespace System.EnumAttribute
{
    public class EnumTitleAttribute : Attribute
    {
        private string title;

        public EnumTitleAttribute(string title)
        {
            this.title = title;
        }

        public string Title { get { return title; } }
    }

    public static class EnumOperator
    {
        /// <summary>
        /// 获取枚举字段上的描述
        /// </summary>
        /// <param name="en">枚举字段</param>
        /// <returns></returns>
        public static string GetTitle(this Enum en)
        {
            if (en == null) throw new ArgumentNullException();

            var filed = en.GetType().GetField(en.ToString());

            if (filed == null) return string.Empty;

            var attrubutes = filed.GetCustomAttributes(false);

            var title = attrubutes.Length == 0 ? null : attrubutes.Where(s => s.GetType() == typeof(EnumTitleAttribute)).FirstOrDefault() as EnumTitleAttribute;

            return title == null ? "" : title.Title;
        }

        /// <summary>
        /// 获取枚举与该枚举的描述
        /// </summary>
        /// <param name="enumType">枚举类型</param>
        /// <returns></returns>
        /// <exception cref="enumType">该类型不是枚举</exception>
        public static IDictionary<Enum, string> GetEnumDictionary(Type enumType)
        {
            if (!enumType.IsEnum) throw new FormatException("当前类型不是枚举");

            IDictionary<Enum, string> dic = new Dictionary<Enum, string>();

            var items = Enum.GetNames(enumType);

            Enum n = null;

            foreach (var item in items)
            {
                n = (Enum)Enum.Parse(enumType, item);
                dic.Add(n, n.GetTitle());
            }

            return dic;
        }

        /// <summary>
        /// 获取枚举与该枚举的描述
        /// </summary>
        /// <typeparam name="T">该类型必须是枚举</typeparam>
        /// <returns></returns>
        /// <exception cref="T">该类型不是枚举</exception>
        public static IDictionary<T, string> GetEnumDictionary<T>()
        {
            var enumType = typeof(T);

            if (!enumType.IsEnum) throw new FormatException("当前类型不是枚举");

            return GetEnumDictionary(enumType).ToDictionary(x => (T)(object)x.Key, y => y.Value);

        }
        /// <summary>
        /// 获取当前枚举的所有项
        /// </summary>
        /// <typeparam name="T">枚举类型</typeparam>
        /// <returns></returns>
        public static IList<T> GetEnums<T>()
        {
            return GetEnums(typeof(T)).Select(x => (T)(object)x).ToList();
        }

        /// <summary>
        /// 获取当前枚举的所有项
        /// </summary>
        /// <param name="enumType"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException">当前type为空</exception>
        /// <exception cref="ArgumentException">当前type不是一个枚举</exception>
        public static IList<Enum> GetEnums(Type enumType)
        {
            if (enumType == null) throw new ArgumentNullException("当前enumType为空");

            if (!enumType.IsEnum) throw new ArgumentException("当前enumType不是一个枚举");

            List<Enum> enums = new List<Enum>();

            var items = Enum.GetNames(enumType);

            Enum n = null;

            foreach (var item in items)
            {
                n = (Enum)Enum.Parse(enumType, item);
                enums.Add(n);
            }

            return enums;
        }
    }
}
