﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Data.Objects;
using System.Data.Objects.DataClasses;
using System.Linq.LinqExtensions;
using System.Data.EntityClient;
namespace Cedrus.Class.Common
{
    public class DBHelper
    {
        /// <summary>
        /// 数据库连接字符串
        /// </summary>
        public static string ConnectionString { get; set; }

        public static string GetSqlConnection
        {
            get
            {
                CedrusEntities db = new CedrusEntities();
                return ((EntityConnection)db.Connection).StoreConnection.ConnectionString;
            }
        }

        /// <summary>
        /// 获取数据库实例对象
        /// </summary>
        public static CedrusEntities GetContext
        {
            get
            {
                CedrusEntities db = new CedrusEntities(ConnectionString);

                return db;
            }
        }

    }

    /// <summary>
    /// 通用数据库处理类，可处理多个表的数据
    /// </summary>
    public class Entity
    {
        public Entity()
        {

        }

        protected CedrusEntities db = new CedrusEntities();

        private List<string> includes = new List<string>();

        /// <summary>
        /// 添加数据
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public bool Add<T>(T data) where T : System.Data.Objects.DataClasses.EntityObject
        {
            db.AddObject(typeof(T).Name, data);

            return db.SaveChanges() > 0;
        }

        /// <summary>
        /// 可同时向多个表中添加记录
        /// </summary>
        /// <typeparam name="EntityType"></typeparam>
        /// <param name="objects"></param>
        /// <returns></returns>
        public bool Add<EntityType>(List<EntityType> objects) where EntityType : System.Data.Objects.DataClasses.EntityObject
        {
            foreach (var o in objects)
            {
                db.AddObject(o.GetType().Name, o);
            }

            return db.SaveChanges() > 0;

        }

        public void EntityRefresh(object obj)
        {
            db.Refresh(RefreshMode.StoreWins, obj);
        }

        /// <summary>
        /// 根据列来获取对象
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="column">列名</param>
        /// <param name="value">列值</param>
        /// <returns></returns>
        public T GetObject<T>(string column, object value) where T : System.Data.Objects.DataClasses.EntityObject
        {
            object obj = null;

            var key = new System.Data.EntityKey()
            {
                EntityContainerName = "CedrusEntities",
                EntityKeyValues = new System.Data.EntityKeyMember[] { new System.Data.EntityKeyMember(column, value) },
                EntitySetName = typeof(T).Name
            };

            if (db.TryGetObjectByKey(key, out obj))
            {
                return (T)obj;
            }

            return default(T);
        }

        /// <summary>
        /// 删除对象
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool Delete(object entity)
        {
            db.DeleteObject(entity);
            return db.SaveChanges() > 0;
        }

        /// <summary>
        /// 根据表名删除表中的数据，无条件
        /// </summary>
        /// <param name="tableName"></param>
        /// <returns></returns>
        public bool Delete(string tableName)
        {
            var eql = @"Select VALUE c from {0} as c ";

            var data = db.CreateQuery<EntityObject>(string.Format(eql, tableName));

            foreach (var d in data)
            {
                db.DeleteObject(d);
            }

            return db.SaveChanges() > 0;
        }

        /// <summary>
        /// 包含的外键表
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="name"></param>
        /// <returns></returns>
        public ObjectQuery<T> IncludeTable<T>(string name) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQuery<T>().Include(name);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="pred">条件</param>
        /// <returns></returns>
        public bool Delete<T>(Expression<Func<T, bool>> pred) where T : System.Data.Objects.DataClasses.EntityObject
        {

            var data = GetQuery<T>(pred);

            foreach (var d in data)
                db.DeleteObject(d);

            return db.SaveChanges() > 0;

        }

        /// <summary>
        /// 删除一个集合
        /// </summary>
        /// <param name="entitys"></param>
        /// <returns></returns>
        public bool DeleteEntitys(List<EntityObject> entitys)
        {
            foreach (var e in entitys)
            {
                db.DeleteObject(e);
            }

            return db.SaveChanges() > 0;
        }


        /// <summary>
        /// 根据相同的条件，从不同的表中删除记录，目前支持的列名与列值的关系为“等于”。不支持其它的
        /// </summary>
        /// <param name="tables">表名集合</param>
        /// <param name="columnName">列名</param>
        /// <param name="value">列值</param>
        public bool Delete(List<string> tables, string columnName, object value)
        {
            var eql = @"Select VALUE c from {0} as c where c.{1}={2}";

            foreach (var t in tables)
            {
                var data = db.CreateQuery<EntityObject>(string.Format(eql, t, columnName, value));

                foreach (var d in data)
                {
                    db.DeleteObject(d);
                }
            }

            return db.SaveChanges() > 0;
        }

        /// <summary>
        /// 编辑
        /// </summary>
        /// <param name="Data"></param>
        /// <returns></returns>
        public bool Edit<T>(T data) where T : System.Data.Objects.DataClasses.EntityObject
        {
            if (data != null)
            {
                var count = db.SaveChanges();

                return count > 0;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 提交更新
        /// </summary>
        /// <returns></returns>
        public bool SaveChange()
        {
            return db.SaveChanges() > 0;
        }

        /// <summary>
        /// 获取记录的集合
        /// </summary>
        /// <returns></returns>
        public ObjectQuery<T> GetQuery<T>() where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetObjectQuery<T>();
        }

        /// <summary>
        /// 获取记录的集合
        /// </summary>
        /// <param name="pred">where条件</param>
        /// <returns></returns>
        public IQueryable<T> GetQuery<T>(Expression<Func<T, bool>> pred) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhere<T>(pred);
        }

        /// <summary>
        /// 获取表中第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <returns></returns>
        public T GetFristOrDefault<T>() where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetObjectQuery<T>().FirstOrDefault();
        }

        /// <summary>
        ///获取表中第一条记录 ，如果没有，返回默认值 
        /// </summary>
        /// <param name="pred">where条件</param>
        /// <returns></returns>
        public T GetFristOrDefault<T>(Expression<Func<T, bool>> pred) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetObjectQuery<T>().FirstOrDefault(pred);
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<T, TResult>(Expression<Func<T, TResult>> selector) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForSingleColumn<T, TResult>(selector).FirstOrDefault();
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="selector">查
        /// 询列表达式</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<T, TResult>(Expression<Func<T, bool>> pred, Expression<Func<T, TResult>> selector) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhereAndSelector<T, TResult>(pred, selector).FirstOrDefault();
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<T, TResult>(string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForSingleColumn<T, TResult>(columnName).FirstOrDefault();
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<T, TResult>(Expression<Func<T, bool>> pred, string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhereAndSelector<T, TResult>(pred, columnName).FirstOrDefault();
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="select">查询列表达式</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<T, TResult>(Expression<Func<T, TResult>> selector) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForSelector<T, TResult>(selector);
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<T, TResult>(string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForSelector<T, TResult>(columnName);
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<T, TResult>(Expression<Func<T, bool>> pred, Expression<Func<T, TResult>> selector) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhereAndSelector<T, TResult>(pred, selector);
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="pred">列类型</param>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<T, TResult>(Expression<Func<T, bool>> pred, string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhereAndSelector<T, TResult>(pred, columnName);
        }

        public void Include(params string[] tables)
        {
            foreach (var t in tables)
            {
                includes.Add(t);
            }
        }

        /// <summary>
        /// 添除关系对象的集合
        /// </summary>
        public void IncludeClear()
        {
            includes.Clear();
        }

        #region 私有方法

        /// <summary>
        /// 生成Lambda表达式，用来创建select中的表达式，如:x=>x.ID
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        private Expression<Func<T, TResult>> GetSelector<T, TResult>(string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            var parm = Expression.Parameter(typeof(T), "s");

            Expression left = Expression.Property(parm, typeof(T).GetProperty(columnName));

            return Expression.Lambda<Func<T, TResult>>(left, parm);
        }

        /// <summary>
        /// 创建ObjectQuery对象
        /// </summary>
        /// <returns></returns>
        private ObjectQuery<T> GetObjectQuery<T>() where T : System.Data.Objects.DataClasses.EntityObject
        {
            var query = db.CreateQuery<T>(typeof(T).Name);

            query.MergeOption = MergeOption.OverwriteChanges;

            foreach (var i in includes)
            {
                query = query.Include(i);
            }

            IncludeClear();

            return query;

        }

        /// <summary>
        /// 创建带有Where条件的IQueryable对象
        /// </summary>
        /// <param name="pred">where条件</param>
        /// <returns></returns>
        private IQueryable<T> GetQueryForWhere<T>(Expression<Func<T, bool>> pred) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetObjectQuery<T>().Where(pred);
        }

        /// <summary>
        /// 创建带有select条件的IQueryable对象
        /// </summary>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForSelector<T, TResult>(Expression<Func<T, TResult>> selector) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetObjectQuery<T>().Select(selector);
        }

        /// <summary>
        /// 创建带有where和select条件的IQueryable对象
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForWhereAndSelector<T, TResult>(Expression<Func<T, bool>> pred, Expression<Func<T, TResult>> selector) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhere<T>(pred).Select(selector);
        }

        /// <summary>
        /// 创建带有select条件的IQueryable<TResult>对象
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForSelector<T, TResult>(string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetObjectQuery<T>().Select(GetSelector<T, TResult>(columnName));
        }

        /// <summary>
        /// 创建带有where和select条件的IQueryable对象
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForWhereAndSelector<T, TResult>(Expression<Func<T, bool>> pred, string columnName) where T : System.Data.Objects.DataClasses.EntityObject
        {
            return GetQueryForWhere<T>(pred).Select(GetSelector<T, TResult>(columnName));
        }
        #endregion



    }

    /// <summary>
    /// 数据库对象通用处理类
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class Entity<T> where T : System.Data.Objects.DataClasses.EntityObject
    {
        protected CedrusEntities db = new CedrusEntities();

        private string tableName = typeof(T).Name;

        private List<string> includes = new List<string>();

        /// <summary>
        /// 添加数据
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public bool Add(T data)
        {
            db.AddObject(tableName, data);

            return db.SaveChanges() > 0;

        }

        /// <summary>
        /// 添加一个集合
        /// </summary> 
        /// <param name="list"></param>
        /// <returns></returns>
        public bool Add(List<T> list)
        {
            foreach (var l in list)
            {
                db.AddObject(tableName, l);
            }

            return db.SaveChanges() > 0;

        }

        /// <summary>
        /// 获取对象
        /// </summary>
        /// <typeparam name="TObject"></typeparam>
        /// <param name="pred"></param>
        /// <returns></returns>
        public TObject GetObject<TObject>(Expression<Func<TObject, bool>> pred)
        {
            return db.CreateQuery<TObject>(typeof(TObject).Name).FirstOrDefault(pred);
        }

        /// <summary>
        /// 获取对象
        /// </summary>
        /// <typeparam name="TObject"></typeparam>
        /// <param name="pred"></param>
        /// <returns></returns>
        public TObject GetObject<TObject>(string column, object value)
        {
            object obj = null;

            var key = new System.Data.EntityKey()
            {
                EntityContainerName = "CedrusEntities",
                EntityKeyValues = new System.Data.EntityKeyMember[] { new System.Data.EntityKeyMember(column, value) },
                EntitySetName = typeof(TObject).Name
            };

            if (db.TryGetObjectByKey(key, out obj))
            {
                return (TObject)obj;
            }

            return default(TObject);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="pred">条件</param>
        /// <returns></returns>
        public bool Delete(Expression<Func<T, bool>> pred)
        {

            var query = GetQuery(pred);

            foreach (var q in query)
            {
                db.DeleteObject(q);
            }

            return db.SaveChanges() > 0;

        }

        /// <summary>
        /// 提交更新
        /// </summary>
        /// <returns></returns>
        public bool SaveChange()
        {
            return db.SaveChanges() > 0;
        }

        /// <summary>
        /// 删除对象
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool Delete(T entity)
        {

            db.DeleteObject(entity);
            return db.SaveChanges() > 0;

        }

        /// <summary>
        /// 删除一个集合
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        public bool Delete(List<T> list)
        {
            if (list == null || list.Count == 0) return false;

            foreach (var l in list)
            {
                db.DeleteObject(l);
            }

            return db.SaveChanges() > 0;

        }

        /// <summary>
        /// 编辑
        /// </summary>
        /// <param name="Data"></param>
        /// <returns></returns>
        public bool Edit(T data)
        {
            if (data != null)
            {
                var count = db.SaveChanges();

                return count > 0;
            }
            else
            {
                return false;
            }
        }


        /// <summary>
        /// 取出最大编号
        /// </summary>
        /// <returns></returns>
        public int GetMaxID()
        {
            return db.CreateQuery<int>(string.Format("select value top(1) c.ID FROM {0} AS c order by c.ID desc", tableName)).FirstOrDefault();
        }

        /// <summary>
        /// 获取记录的集合
        /// </summary>
        /// <returns></returns>
        public IQueryable<T> GetQuery()
        {
            return GetObjectQuery();
        }

        /// <summary>
        /// 包含的外键表
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="name"></param>
        /// <returns></returns>
        public ObjectQuery<T> IncludeTable(string name)
        {
            return ((ObjectQuery<T>)GetQuery()).Include(name);
        }

        /// <summary>
        /// 获取记录的集合
        /// </summary>
        /// <param name="pred">where条件</param>
        /// <returns></returns>
        public IQueryable<T> GetQuery(Expression<Func<T, bool>> pred)
        {
            return GetQueryForWhere(pred);
        }

        /// <summary>
        /// 获取表中第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <returns></returns>
        public T GetFristOrDefault()
        {
            return GetObjectQuery().FirstOrDefault();
        }

        /// <summary>
        ///获取表中第一条记录 ，如果没有，返回默认值 
        /// </summary>
        /// <param name="pred">where条件</param>
        /// <returns></returns>
        public T GetFristOrDefault(Expression<Func<T, bool>> pred)
        {
            return GetObjectQuery().FirstOrDefault(pred);
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<TResult>(Expression<Func<T, TResult>> selector)
        {
            return GetQueryForSingleColumn(selector).FirstOrDefault();
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<TResult>(Expression<Func<T, bool>> pred, Expression<Func<T, TResult>> selector)
        {
            return GetQueryForWhereAndSelector(pred, selector).FirstOrDefault();
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<TResult>(string columnName)
        {
            return GetQueryForSingleColumn<TResult>(columnName).FirstOrDefault();
        }

        /// <summary>
        /// 获取单一列的第一条记录，如果没有，返回默认值 
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public TResult GetFristOrDefaultForSingleColumn<TResult>(Expression<Func<T, bool>> pred, string columnName)
        {
            return GetQueryForWhereAndSelector<TResult>(pred, columnName).FirstOrDefault();
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="select">查询列表达式</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<TResult>(Expression<Func<T, TResult>> selector)
        {
            return GetQueryForSelector(selector);
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<TResult>(string columnName)
        {
            return GetQueryForSelector<TResult>(columnName);
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<TResult>(Expression<Func<T, bool>> pred, Expression<Func<T, TResult>> selector)
        {
            return GetQueryForWhereAndSelector<TResult>(pred, selector);
        }

        /// <summary>
        /// 获取某单一列的记录列表
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="pred">列类型</param>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        public IQueryable<TResult> GetQueryForSingleColumn<TResult>(Expression<Func<T, bool>> pred, string columnName)
        {
            return GetQueryForWhereAndSelector<TResult>(pred, columnName);
        }

        public void Include(string name)
        {
            includes.Add(name);
        }

        #region 私有方法

        /// <summary>
        /// 生成Lambda表达式，用来创建select中的表达式，如:x=>x.ID
        /// </summary>
        /// <typeparam name="TResult">列类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        private Expression<Func<T, TResult>> GetSelector<TResult>(string columnName)
        {
            var parm = Expression.Parameter(typeof(T), "s");

            Expression left = Expression.Property(parm, typeof(T).GetProperty(columnName));

            return Expression.Lambda<Func<T, TResult>>(left, parm);
        }

        /// <summary>
        /// 创建ObjectQuery对象
        /// </summary>
        /// <returns></returns>
        private ObjectQuery<T> GetObjectQuery()
        {
            var query = db.CreateQuery<T>(tableName);

            query.MergeOption = MergeOption.PreserveChanges;

            foreach (var i in includes)
            {
                query = query.Include(i);
            }

            return query;

        }

        /// <summary>
        /// 创建带有Where条件的IQueryable对象
        /// </summary>
        /// <param name="pred">where条件</param>
        /// <returns></returns>
        private IQueryable<T> GetQueryForWhere(Expression<Func<T, bool>> pred)
        {
            return GetObjectQuery().Where(pred);
        }

        /// <summary>
        /// 创建带有select条件的IQueryable对象
        /// </summary>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForSelector<TResult>(Expression<Func<T, TResult>> selector)
        {
            return GetObjectQuery().Select(selector);
        }

        /// <summary>
        /// 创建带有where和select条件的IQueryable对象
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="selector">查询列表达式</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForWhereAndSelector<TResult>(Expression<Func<T, bool>> pred, Expression<Func<T, TResult>> selector)
        {
            return GetQueryForWhere(pred).Select(selector);
        }

        /// <summary>
        /// 创建带有select条件的IQueryable对象
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForSelector<TResult>(string columnName)
        {
            return GetObjectQuery().Select(GetSelector<TResult>(columnName));
        }

        /// <summary>
        /// 创建带有where和select条件的IQueryable对象
        /// </summary>
        /// <typeparam name="TResult">返回类型</typeparam>
        /// <param name="pred">where条件</param>
        /// <param name="columnName">列名</param>
        /// <returns></returns>
        private IQueryable<TResult> GetQueryForWhereAndSelector<TResult>(Expression<Func<T, bool>> pred, string columnName)
        {
            return GetQueryForWhere(pred).Select(GetSelector<TResult>(columnName));
        }
        #endregion

    }


}
