﻿using System.Web;
using System.Web.Routing;

namespace System.Web.Routing.WebRouting
{
    public interface IRoutableHttpHandler
    {
        RequestContext RequestContext { set; }
    }

    public class RouteHandler<T> : IRouteHandler where T : IHttpHandler, new()
    {
        #region IRouteHandler Members

        public IHttpHandler GetHttpHandler(RequestContext requestContext)
        {
            var handler = new T();

            if (handler is IRoutableHttpHandler)
                ((IRoutableHttpHandler)handler).RequestContext = requestContext;

            return handler;
        }

        #endregion
    }

    public abstract class VirtualRouteHttpHandler : IHttpHandler, IRoutableHttpHandler
    {
        public RequestContext RequestContext { protected get; set; }

        /// <summary>
        /// Gets a parameterized virtual path.
        /// </summary>
        /// <param name="values">The values.</param>
        /// <returns></returns>
        protected string GetVirtualPath(object values)
        {
            return GetVirtualPath(null, values);
        }

        /// <summary>
        /// Gets a parameterized virtual path.
        /// </summary>
        /// <param name="routeName">The route name.</param>
        /// <param name="values">The values.</param>
        /// <returns></returns>
        protected string GetVirtualPath(string routeName, object values)
        {
            return (RequestContext != null)
                ? RouteTable.Routes.GetVirtualPath(RequestContext, routeName, new RouteValueDictionary(values)).VirtualPath
                : null;
        }

        /// <summary>
        /// Gets a route value.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        protected object RouteValue(string key)
        {
            lock (RequestContext.RouteData)
                return RequestContext.RouteData.Values[key];
        }

        #region IHttpHandler Members

        public bool IsReusable
        {
            get { return true; }
        }
        protected abstract void DoProcessRequest(HttpContext context);
        public void ProcessRequest(HttpContext context)
        {
            DoProcessRequest(context);
        }

        #endregion

        #region IRoutableHttpHandler Members

        RequestContext IRoutableHttpHandler.RequestContext
        {
            set { RequestContext = value; }
        }

        #endregion
    }
}
