﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Threading;
using System.Web.Caching;

namespace System.Threading.AutoThread
{
    class AutoThreadParam
    {
        internal string Name { get; set; }

        internal int WaitMinute { get; set; }

        internal System.Threading.Thread AutoThread { get; set; }

        internal ParameterizedThreadStart Method { get; set; }

        internal bool IsRunning { get; set; }

        internal object Obj { get; set; }
    }

    class AutoExcuenMethod
    {
        private AutoThreadParam parm;

        private int minute = 1000 * 60;

        internal AutoExcuenMethod(AutoThreadParam parm)
        {
            this.parm = parm;
        }

        internal void Create(object obj)
        {
            while (parm.IsRunning)
            {
                parm.Method.Invoke(obj);

                try
                {
                    System.Threading.Thread.Sleep(parm.WaitMinute * minute);
                }
                catch
                {

                }
            }
        }
    }

    public static class AutoThread
    {
        private static List<AutoThreadParam> threads;

        static AutoThread()
        {
            threads = new List<AutoThreadParam>();
        }

        public static int Count { get { return threads.Count; } }

        public static void Create(string name, int waitMinute, ParameterizedThreadStart method, object obj)
        {
            if (Contain(name)) throw new Exception("Current name is excits");

            if (waitMinute == 0) throw new Exception("时间应大于0");

            var thread = new AutoThreadParam()
           {
               Method = method,
               Name = name,
               WaitMinute = waitMinute,
               IsRunning = true,
               Obj = obj
           };

            ParameterizedThreadStart start = new ParameterizedThreadStart(new AutoExcuenMethod(thread).Create);

            thread.AutoThread = new System.Threading.Thread(start);

            thread.AutoThread.IsBackground = true;

            thread.AutoThread.Start(obj);

            threads.Add(thread);
        }

        public static bool IsAlive(string name)
        {
            CheckExcitsName(name);
            return GetThread(name).IsAlive;
        }

        /// <summary>
        /// 如果线程应意外而终止，则可重新唤起此线程，如果程序依然在运行，则什么都不做
        /// </summary>
        /// <param name="name">线程的名称</param>
        public static void ReStart(string name)
        {
            if (!IsAlive(name))
            {
                var parms = GetThreadParms(name);
                var minutes = parms.WaitMinute;
                var methord = parms.Method;
                var obj = parms.Obj;

                try
                {
                    Stop(name);

                    Create(name, minutes, methord, obj);
                }
                finally
                {
                    
                }
            }
        }

        public static string[] GetAllThreadName()
        {
            return threads.Select(x => x.Name).ToArray();
        }

        public static void Stop(string name)
        {
            CheckExcitsName(name);

            var thread = GetThreadParms(name);
            //  thread.Abort();
            thread.IsRunning = false;
            thread.WaitMinute = 0;
            Interrupt(name);
            threads.RemoveAll(s => s.Name == name);
        }

        public static bool Contain(string name)
        {
            return threads.Where(s => s.Name == name).Count() > 0;
        }

        public static int GetMinute(string name)
        {
            CheckExcitsName(name);

            return GetThreadParms(name).WaitMinute;
        }

        public static void SetMinute(string name, int minute)
        {
            CheckExcitsName(name);

            var parm = GetThreadParms(name);

            parm.WaitMinute = minute;

            Interrupt(name);

        }

        private static ParameterizedThreadStart GetThreadStart(string name)
        {
            CheckExcitsName(name);

            return GetThreadParms(name).Method;
        }

        private static void SetThreadStart(string name, ParameterizedThreadStart start)
        {
            CheckExcitsName(name);

            var parm = GetThreadParms(name);

            parm.Method = start;
        }

        private static void Interrupt(string name)
        {
            CheckExcitsName(name);
            var thread = GetThread(name);
            if (thread.ThreadState == ThreadState.WaitSleepJoin)
                thread.Interrupt();
        }

        private static System.Threading.Thread GetThread(string name)
        {
            CheckExcitsName(name);

            return GetThreadParms(name).AutoThread;
        }

        private static AutoThreadParam GetThreadParms(string name)
        {
            return threads.Where(s => s.Name == name).FirstOrDefault();
        }

        private static void CheckExcitsName(string name)
        {
            if (string.IsNullOrEmpty(name) || !Contain(name))
            {
                throw new ArgumentException();
            }
        }
    }


}