﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LuceneLib.Class;

namespace LuceneTest
{
    class Program
    {
        static string indexPath = "index";

        static void Main(string[] args)
        {
            //生成索引
            BulidIndex();

            //检索
            Search();

            //检索排序
            SearchAndOrderBy();

            Console.ReadKey();
        }

        static void BulidIndex()
        {
            if (System.IO.Directory.Exists(indexPath))
            {
                System.IO.Directory.Delete(indexPath, true);
            }

            foreach (var item in LuceneLib.Class.BulidIndex.WriteIndex(indexPath, GetDataList()))
            {
                Console.WriteLine("当前索引记录:" + item.ToString());
            }
        }

        static void Search()
        {
            int total = 0;
            //1)	字符串检索
            var stringSearch = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "Title", OperatorType = Operator.Like, Value = "计算机" }, null, 1, 10, out total, 1000);
            Pint("stringSearch", stringSearch);

            //2)	时间、整形检索
            var intSearch = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "ID", OperatorType = Operator.Between, Value = "0,1000" }, null, 1, 10, out total, 1000);
            Pint("intSearch", intSearch);

            var startTime = DateTime.Now.AddYears(-5).Ticks.ToString();
            var endTime = DateTime.Now.Ticks.ToString();
            var timeSearch = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "Publish_long", OperatorType = Operator.Between, Value = startTime + "," + endTime }, null, 1, 10, out total, 1000);
            Pint("timeSearch", timeSearch);

            //3)	全词匹配
            var stringTermSearch = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "Title", OperatorType = Operator.Equal, Value = "人月神话" }, null, 1, 10, out total, 1000);
            Pint("stringTermSearch", stringTermSearch);

            //4)	多条件检索
            var searchObjects = new List<SearchObject>();
            searchObjects.Add(new SearchObject() { Column = "Title", OperatorType = Operator.Like, Value = "人月神话", Logic = Logic.or });
            searchObjects.Add(new SearchObject() { Column = "Author", OperatorType = Operator.Like, Value = "李丽", Logic = Logic.or });
            var mutiQuerySearch = LuceneLib.Class.LuneceSearch.Search(indexPath, searchObjects, null, 1, 10, out total, 1000);
            Pint("mutiQuerySearch", mutiQuerySearch);

            //5)	多索引检索
            var mutiSearch = LuceneLib.Class.LuneceSearch.MultiSearch(new string[] { indexPath }, searchObjects, null, 1, 10, out total, 1000);
            Pint("mutiSearch", mutiSearch);

            //6)	多索引并行检索
            var parallelMutiSearch = LuceneLib.Class.LuneceSearch.ParallelMultiSearcher(new string[] { indexPath }, searchObjects, null, 1, 10, out total, 1000);
            Pint("parallelMutiSearch", parallelMutiSearch);

        }

        static void SearchAndOrderBy()
        {
            int total = 0;
            //字符串排序
            var stringSearchOrderby = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "Title", OperatorType = Operator.Like, Value = "计算机" }, new OrderBy() { Coloumn = "Title", IsDesc = true, OrderByType = OrderByColumn.STRING }, 1, 10, out total, 1000);
            Pint("stringSearchOrderby", stringSearchOrderby);

            //时间排序
            var timeSearchOrderby = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "Title", OperatorType = Operator.Like, Value = "计算机" }, new OrderBy() { Coloumn = "Publish_sort", IsDesc = true, OrderByType = OrderByColumn.LONG }, 1, 10, out total, 1000);
            Pint("timeSearchOrderby", timeSearchOrderby);


            //ID排序 
            var idSearchOrderby = LuceneLib.Class.LuneceSearch.Search(indexPath, new SearchObject() { Column = "Title", OperatorType = Operator.Like, Value = "计算机" }, new OrderBy() { Coloumn = "ID", IsDesc = true, OrderByType = OrderByColumn.LONG }, 1, 10, out total, 1000);
            Pint("idSearchOrderby", idSearchOrderby);

        }

        static void Pint(string name, List<Result> result)
        {
            Console.WriteLine(name + "," + result.Count);
            foreach (var item in result)
            {
                Console.WriteLine(LuceneLib.Class.LuneceSearch.GetValue(indexPath, item.DocID, "Title"));
            }
            Console.WriteLine();
        }

        #region 索引相关
        private static DataObject GetDataObject(string columnName, object value, DataTypes dataType)
        {
            return new DataObject()
            {
                Column = columnName,
                Value = value,
                IsStore = true,
                IsSort = true,
                DataType = dataType
            };
        }

        private static IEnumerable<DataObject[]> GetDataList()
        {
            List<BookObj> bookObj = new List<BookObj>()
            {
                new BookObj(){ Author="王明", ID=1, Publish=new DateTime(2011,3,5), Title="计算机理论"},
                new BookObj(){ Author="李丽", ID=2, Publish=new DateTime(2001,1,2), Title="C#编程"},
                new BookObj(){ Author="巍巍", ID=3, Publish=new DateTime(2009,9,5), Title="人月神话"},
            };

            foreach (var item in bookObj)
            {
                List<DataObject> dataList = new List<DataObject>();

                dataList.Add(GetDataObject("ID", item.ID, DataTypes.Int));
                dataList.Add(GetDataObject("Title", item.Title, DataTypes.String));
                dataList.Add(GetDataObject("Author", item.Author, DataTypes.String));
                dataList.Add(GetDataObject("Publish", item.Publish, DataTypes.DateTime));
                dataList.Add(GetDataObject("Publish_long", item.Publish.Ticks, DataTypes.Int));
                yield return dataList.ToArray();
            }
        }

        #endregion

    }

    public class BookObj
    {
        public int ID { get; set; }

        public string Title { get; set; }

        public string Author { get; set; }

        public DateTime Publish { get; set; }
    }
}
